/*******************************************************************************

NAME            eseriesKernelInterfaceTests.cc
VERSION         %version:  1 %
UPDATE DATE     %date_modified: Mon Dec 19 14:46:42 2016 %
PROGRAMMER      %created_by:    dharmana %

        Copyright 2016-2017 NetApp, Inc. All Rights Reserved.

*******************************************************************************/
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>

#include "unittest++/UnitTest++.h"
#include "eseries_kernel_interface.h"

/**
 * Test entry for eseries-kernel-interface test suites
 */
int main(int argc, char *argv[])
{
    int status = 0;
    bool list_tests = false;

    // Parse command line
    for (int i = 0; i < argc; ++i)
    {
        if (strncmp(argv[i], "--list", strlen("--list")) == 0)
            list_tests = true;
    }

    if (list_tests)
    {
        // Dump all tests from all suites
        UnitTest::TestList list = UnitTest::Test::GetTestList();
        for (UnitTest::Test* test = list.GetHead(); test; test = test->next)
            std::cout << test->m_details.suiteName << "::" << test->m_details.testName <<std::endl;
    }
    else
    {
        // Run all tests from all suites
        status = UnitTest::RunAllTests();
    }

    return status;
}

/**
 * This suite includes tests for eseries-kernel-interface module
 */
SUITE(eseriesKernelInterfaceTests)
{
    char     fakeRegionSignature[PR_SIGNATURE_LEN]      = "FAKE_PERSISTENT REGION";
    uint32_t fakeRegionVersion                          = -1;
    uint32_t fakeErrorRegionVersion                     = -1;
    char     fakeErrorRegionSignature[PR_SIGNATURE_LEN] = "FAKE_ERROR REGION";

    char     origRegionSignature[PR_SIGNATURE_LEN];
    uint32_t origRegionVersion;
    uint32_t origErrorRegionVersion;
    char     origErrorRegionSignature[PR_SIGNATURE_LEN];

    struct persistent_reg_imtdata imtData;
    int fd;

    TEST(PersistentRegionTestSetup)
    {
        fd = open(EKI_DEV_PATH, O_RDONLY, 0);
        CHECK(fd != -1);

        /* read in-memory version/signature global values and save them */
        imtData.op = persistent_reg_imtdata::READ_GLOBAL;

        /* header region version */
        imtData.var = persistent_reg_imtdata::REG_VER;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        origRegionVersion =imtData.data.version;

        /* header region signature */
        imtData.var = persistent_reg_imtdata::REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        memcpy(origRegionSignature, imtData.data.signature, sizeof(imtData.data.signature));

        /* error logging region version */
        imtData.var = persistent_reg_imtdata::ERR_REG_VER;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        origErrorRegionVersion =imtData.data.version;

        /* error logging region signature */
        imtData.var = persistent_reg_imtdata::ERR_REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        memcpy(origErrorRegionSignature, imtData.data.signature, sizeof(imtData.data.signature));
    }

    TEST(PersistentRegionVersionMismatch)
    {
        /* write fake version to the header region global variable */
        imtData.op = persistent_reg_imtdata::WRITE_GLOBAL;
        imtData.var = persistent_reg_imtdata::REG_VER;
        imtData.data.version = fakeRegionVersion;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* initialize persistent region */
        imtData.op = persistent_reg_imtdata::CALL_INIT;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* read version from the header region */
        imtData.op = persistent_reg_imtdata::READ_REGION;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        CHECK_EQUAL(imtData.data.version, fakeRegionVersion);
    }

    TEST(PersistentRegionSignatureMismatch)
    {
        /* write fake signature to the header region global variable */
        imtData.op = persistent_reg_imtdata::WRITE_GLOBAL;
        imtData.var = persistent_reg_imtdata::REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        memcpy(imtData.data.signature, fakeRegionSignature, sizeof(fakeRegionSignature));
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* initialize persistent region */
        imtData.op = persistent_reg_imtdata::CALL_INIT;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* read signature from the header region */
        imtData.op = persistent_reg_imtdata::READ_REGION;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        CHECK_ARRAY_EQUAL(fakeRegionSignature, imtData.data.signature, PR_SIGNATURE_LEN);
    }

    TEST(PersistentErrorRegionVersionMismatch)
    {
        /* write fake version to the logging region global variable */
        imtData.op = persistent_reg_imtdata::WRITE_GLOBAL;
        imtData.var = persistent_reg_imtdata::ERR_REG_VER;
        imtData.data.version = fakeErrorRegionVersion;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* initialize persistent region */
        imtData.op = persistent_reg_imtdata::CALL_INIT;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* read version from the loggin region */
        imtData.op = persistent_reg_imtdata::READ_REGION;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        CHECK_EQUAL(imtData.data.version, fakeErrorRegionVersion);
    }

    TEST(PersistentErrorRegionSignatureMismatch)
    {
        /* write fake signature to the logging region global variable*/
        imtData.op = persistent_reg_imtdata::WRITE_GLOBAL;
        imtData.var = persistent_reg_imtdata::ERR_REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        memcpy(imtData.data.signature, fakeRegionSignature, sizeof(fakeRegionSignature));
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* initialize persistent region */
        imtData.op = persistent_reg_imtdata::CALL_INIT;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* read signature from the logging region */
        imtData.op = persistent_reg_imtdata::READ_REGION;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        CHECK_ARRAY_EQUAL(fakeRegionSignature, imtData.data.signature, PR_SIGNATURE_LEN);
    }

    TEST(PersistentRegionTestTearDown)
    {
        /* write originial version and signature into the header and error logging region
         * global variables */
        imtData.op = persistent_reg_imtdata::WRITE_GLOBAL;

        /* orig header version*/
        imtData.var = persistent_reg_imtdata::REG_VER;
        imtData.data.version = origRegionVersion;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* orig header signature*/
        imtData.var = persistent_reg_imtdata::REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        memcpy(imtData.data.signature, origRegionSignature, sizeof(origRegionSignature));
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* orig logging version */
        imtData.var = persistent_reg_imtdata::ERR_REG_VER;
        imtData.data.version = origErrorRegionVersion;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /* orig logging signature*/
        imtData.var = persistent_reg_imtdata::ERR_REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        memcpy(imtData.data.signature, origErrorRegionSignature, sizeof(origErrorRegionSignature));
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

        /********* initialize persistent region ***********/
        imtData.op = persistent_reg_imtdata::CALL_INIT;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));

         /* read version and signature from the header and logging regions
          * and compare them to original version and signature */
        imtData.op = persistent_reg_imtdata::READ_REGION;

        /* header region version */
        imtData.var = persistent_reg_imtdata::REG_VER;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));
        CHECK_EQUAL(imtData.data.version, origRegionVersion);

        /* header region signature */
        imtData.var = persistent_reg_imtdata::REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));
        CHECK_ARRAY_EQUAL(origRegionSignature, imtData.data.signature, PR_SIGNATURE_LEN);

        /* logging region version */
        imtData.var = persistent_reg_imtdata::ERR_REG_VER;
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));
        CHECK_EQUAL(imtData.data.version, origErrorRegionVersion);

        /* logging region signature */
        imtData.var = persistent_reg_imtdata::ERR_REG_SIG;
        memset(imtData.data.signature, '\0', PR_SIGNATURE_LEN);
        CHECK(!ioctl(fd, EKI_IOCTL_PER_REG_IMT, &imtData));
        CHECK_ARRAY_EQUAL(origErrorRegionSignature, imtData.data.signature, PR_SIGNATURE_LEN);

        close(fd);
    }
}
